@@ -19,7 +19,10 @@
 # THE SOFTWARE.
 #
 
-"""Implements a simple forwarding policy, to transform or replace recipients."""
+"""Implements a simple forwarding policy, to transform or replace
+recipients.
+
+"""
 
 from __future__ import absolute_import
 
@@ -45,10 +48,10 @@ class Forward(QueuePolicy):
         """Adds a mapping rule.
 
         :param pattern: Pattern to check recipient against.
-        :type pattern: string or :class:`re.RegexObject`
+        :type pattern: :py:obj:`str` or :class:`re.RegexObject`
         :param repl: Replacement for ``pattern`` matches, as described by
                      :func:`re.sub()`.
-        :type repl: string or function
+        :type repl: :py:obj:`str` or function
         :param count: Max number of replacements per recipient string.
 
         """
