@@ -29,6 +29,7 @@ SMTP queues will retry messages after certain periods of time.
 from __future__ import absolute_import
 
 import time
+import bisect
 from itertools import imap, izip, repeat, chain
 
 import gevent
@@ -132,9 +133,10 @@ class QueueStorage(object):
         they were received, the storage mechanism needs a way to wait until it
         is notified that a new message has been stored.
 
-        :returns: A tuple of the timestamp and unique identifier string of a new
-                  message in storage. If ``None`` is returned indicating a
-                  timeout, this method is simply called again.
+        :returns: An iterable or generator producing tuples with the timestamp
+                  and unique identifier string of a new message in storage. When
+                  the iterable or generator is exhausted, :meth:`.wait` is
+                  simply called again.
 
         """
         raise NotImplementedError()
@@ -174,6 +176,8 @@ class Queue(Greenlet):
         self.bounce_factory = bounce_factory or Bounce
         self.wake = Event()
         self.queued = []
+        self.active_ids = set()
+        self.queued_ids = set()
         self.queued_lock = Semaphore(1)
         self.queue_policies = []
         self._use_pool('store_pool', store_pool)
@@ -244,13 +248,11 @@ class Queue(Greenlet):
         return pool.spawn(func, *args, **kwargs)
 
     def _add_queued(self, entry):
-        for i, info in enumerate(self.queued):
-            if info[0] > entry[0]:  # [0] is the timestamp.
-                self.queued.insert(i, entry)
-                break
-        else:
-            self.queued.append(entry)
-        self.wake.set()
+        timestamp, id = entry
+        if id not in self.queued_ids | self.active_ids:
+            bisect.insort(self.queued, entry)
+            self.queued_ids.add(id)
+            self.wake.set()
 
     def enqueue(self, envelope):
         """Drops a new message in the queue for delivery. The first delivery
@@ -271,6 +273,7 @@ class Queue(Greenlet):
         for env, id in results:
             if not isinstance(id, BaseException):
                 if self.relay:
+                    self.active_ids.add(id)
                     self._pool_spawn('relay', self._attempt, id, env, 0)
             elif not isinstance(id, QueueError):
                 raise id  # Re-raise exceptions that are not QueueError.
@@ -280,13 +283,18 @@ class Queue(Greenlet):
         for entry in self.store.load():
             self._add_queued(entry)
 
+    def _remove(self, id):
+        self.store.remove(id)
+        self.queued_ids.discard(id)
+        self.active_ids.discard(id)
+
     def _bounce(self, envelope, reply):
         bounce = self.bounce_factory(envelope, reply)
         if bounce:
             return self.enqueue(bounce)
 
     def _perm_fail(self, id, envelope, reply):
-        self._pool_spawn('store', self.store.remove, id)
+        self._pool_spawn('store', self._remove, id)
         if envelope.sender:  # Can't bounce to null-sender.
             self._pool_spawn('bounce', self._bounce, envelope, reply)
 
@@ -299,6 +307,7 @@ class Queue(Greenlet):
         else:
             when = time.time() + wait
             self.store.set_timestamp(id, when)
+            self.active_ids.discard(id)
             self._add_queued((when, id))
 
     def _attempt(self, id, envelope, attempts):
@@ -314,10 +323,14 @@ class Queue(Greenlet):
             self._pool_spawn('store', self._retry_later, id, envelope, reply)
             raise
         else:
-            self._pool_spawn('store', self.store.remove, id)
+            self._pool_spawn('store', self._remove, id)
 
     def _dequeue(self, id):
-        envelope, attempts = self.store.get(id)
+        try:
+            envelope, attempts = self.store.get(id)
+        except KeyError:
+            return
+        self.active_ids.add(id)
         self._pool_spawn('relay', self._attempt, id, envelope, attempts)
 
     def _check_ready(self, now):
@@ -331,15 +344,15 @@ class Queue(Greenlet):
                 break
         if last_i > 0:
             self.queued = self.queued[last_i:]
+            self.queued_ids = set([id for timestamp, id in self.queued])
 
     def _wait_store(self):
         while True:
             try:
-                entry = self.store.wait()
+                for entry in self.store.wait():
+                    self._add_queued(entry)
             except NotImplementedError:
                 return
-            if entry:
-                self._add_queued(entry)
 
     def _wait_ready(self, now):
         try:
@@ -370,6 +383,14 @@ class Queue(Greenlet):
         finally:
             self.queued_lock.release()
 
+    def kill(self):
+        """This method is used by |Queue| and |Queue|-like objects to properly
+        end any associated services (such as running :class:`~gevent.Greenlet`
+        threads) and close resources.
+
+        """
+        super(Queue, self).kill()
+
     def _run(self):
         if not self.relay:
             return
