@@ -1,5 +1,6 @@
 
 import unittest
+from email.encoders import encode_base64
 
 from mox import MoxTestBase, IsA
 from gevent.socket import socket, error as socket_error
@@ -206,18 +207,21 @@ class TestSmtpRelayClient(MoxTestBase):
     def test_send_envelope(self):
         result = self.mox.CreateMock(AsyncResult)
         env = Envelope('sender@example.com', ['rcpt@example.com'])
-        env.parse('From: sender@example.com\r\n\r\ntest test\r\n')
+        env.parse('From: sender@example.com\r\n\r\ntest test \x81\r\n')
+        self.sock.sendall('EHLO test\r\n')
+        self.sock.recv(IsA(int)).AndReturn('250-Hello\r\n250 8BITMIME\r\n')
         self.sock.sendall('MAIL FROM:<sender@example.com>\r\n')
         self.sock.recv(IsA(int)).AndReturn('250 Ok\r\n')
         self.sock.sendall('RCPT TO:<rcpt@example.com>\r\n')
         self.sock.recv(IsA(int)).AndReturn('250 Ok\r\n')
         self.sock.sendall('DATA\r\n')
         self.sock.recv(IsA(int)).AndReturn('354 Go ahead\r\n')
-        self.sock.sendall('From: sender@example.com\r\n\r\ntest test\r\n.\r\n')
+        self.sock.sendall('From: sender@example.com\r\n\r\ntest test \x81\r\n.\r\n')
         self.sock.recv(IsA(int)).AndReturn('250 Ok\r\n')
         self.mox.ReplayAll()
-        client = SmtpRelayClient(None, self.queue, socket_creator=self._socket_creator)
+        client = SmtpRelayClient(None, self.queue, socket_creator=self._socket_creator, ehlo_as='test')
         client._connect()
+        client._ehlo()
         client._send_envelope(result, env)
 
     def test_send_envelope_badpipeline(self):
@@ -291,6 +295,35 @@ class TestSmtpRelayClient(MoxTestBase):
         with self.assertRaises(ConnectionLost):
             client._send_envelope(result, env)
 
+    def test_send_envelope_conversion(self):
+        result = self.mox.CreateMock(AsyncResult)
+        env = Envelope('sender@example.com', ['rcpt@example.com'])
+        env.parse('From: sender@example.com\r\n\r\ntest test \x81\r\n')
+        self.sock.sendall('EHLO test\r\n')
+        self.sock.recv(IsA(int)).AndReturn('250-Hello\r\n250 PIPELINING\r\n')
+        self.sock.sendall('MAIL FROM:<sender@example.com>\r\nRCPT TO:<rcpt@example.com>\r\nDATA\r\n')
+        self.sock.recv(IsA(int)).AndReturn('250 Ok\r\n250 Ok\r\n354 Go ahead\r\n')
+        self.sock.sendall('From: sender@example.com\r\nContent-Transfer-Encoding: base64\r\n\r\ndGVzdCB0ZXN0IIENCg==\n\r\n.\r\n')
+        self.sock.recv(IsA(int)).AndReturn('250 Ok\r\n')
+        self.mox.ReplayAll()
+        client = SmtpRelayClient(None, self.queue, socket_creator=self._socket_creator, ehlo_as='test', binary_encoder=encode_base64)
+        client._connect()
+        client._ehlo()
+        client._send_envelope(result, env)
+
+    def test_send_envelope_conversion_failure(self):
+        result = self.mox.CreateMock(AsyncResult)
+        env = Envelope('sender@example.com', ['rcpt@example.com'])
+        env.parse('From: sender@example.com\r\n\r\ntest test \x81\r\n')
+        self.sock.sendall('EHLO test\r\n')
+        self.sock.recv(IsA(int)).AndReturn('250-Hello\r\n250 PIPELINING\r\n')
+        result.set_exception(IsA(PermanentRelayError))
+        self.mox.ReplayAll()
+        client = SmtpRelayClient(None, self.queue, socket_creator=self._socket_creator, ehlo_as='test')
+        client._connect()
+        client._ehlo()
+        client._send_envelope(result, env)
+
     def test_disconnect(self):
         self.sock.sendall('QUIT\r\n')
         self.sock.recv(IsA(int)).AndReturn('221 Goodbye\r\n')
